@@ -5,17 +5,19 @@ import logging
 from io import BytesIO
 from datetime import datetime
 import json
+import traceback
 
 from urllib.request import urlopen
 from email.mime.text import MIMEText
 from email.mime.application import MIMEApplication
 from email.mime.multipart import MIMEMultipart
 
-from croniter import croniter
 from flask.wrappers import Response
 
 import jwt
 import boto3
+import botocore
+from botocore.exceptions import NoCredentialsError
 import flask
 import requests
 
@@ -44,7 +46,7 @@ if storage_name == 's3':
 elif storage_name == 'ceph':
     ACCESS_KEY_ID = os.getenv('ACCESS_KEY_ID')
     SECRET_ACCESS_KEY = os.getenv('SECRET_ACCESS_KEY')
-    BUCKET_NAME = os.getenv('BUCKET_NAME')
+    BUCKET_NAME = os.getenv('BUCKET_NAME', 'analytics-apps')
     ENDPOINT_URL = os.getenv('ENDPOINT_URL')
 else:
     logger.info("No storages are found")
@@ -175,8 +177,9 @@ def get_epoc_from_datetime_string(str_datetime):
 
 def get_result_by_run(run_id, field=None, default_value=None):
     try:
-        run_id= f'{PLATFORM_ROUTE}/{run_id}/json/{run_id}'
+        # run_id= f'{PLATFORM_ROUTE}/{run_id}/json/{run_id}'
         if storage_name == 's3':
+            run_id= f'{PLATFORM_ROUTE}/{run_id}/json/{run_id}'
             s3 = get_s3_client()
             res_object = s3.get_object(Bucket=BUCKET_NAME,Key=run_id)
             serializedObject = res_object['Body'].read()
@@ -185,7 +188,8 @@ def get_result_by_run(run_id, field=None, default_value=None):
                 result = result.get(field, default_value)
             return result
         elif storage_name == 'ceph':
-            s3 = get_s3_ceph()
+            run_id= f'{APP_ID.lower()}/{run_id}/json/{run_id}'
+            s3 = get_ceph_resource()
             data = BytesIO()
             res_object = s3.Bucket(BUCKET_NAME).download_fileobj(Fileobj=data,Key=run_id)   
             res_object = data.getvalue()
@@ -224,13 +228,21 @@ def get_s3_client():
                         aws_secret_access_key=SECRET_ACCESS_KEY)
     
 
-def get_s3_ceph():
+def get_ceph_resource():
     return boto3.resource('s3',
                             endpoint_url=ENDPOINT_URL,
                             aws_access_key_id=ACCESS_KEY_ID,
                             aws_secret_access_key=SECRET_ACCESS_KEY)
 
 
+def get_ceph_client():
+    return boto3.client('s3',
+                          endpoint_url=ENDPOINT_URL,
+                          aws_access_key_id=ACCESS_KEY_ID,
+                          aws_secret_access_key=SECRET_ACCESS_KEY)
+
+
+
 def send_email(subject, from_email, to_emails, body, attachment=None):
     message = MIMEMultipart()
     message['Subject'] = subject
@@ -258,7 +270,7 @@ def send_email(subject, from_email, to_emails, body, attachment=None):
     return resp
 
 
-def upload_to_s3(content, location):
+def upload_to_storage(content, location):
     '''
     :param: content: bytes
     :param: location: str
@@ -278,10 +290,25 @@ def upload_to_s3(content, location):
             pass
     elif storage_name == 'ceph':
         try:
-            s3 = boto3.resource('s3',
-                                endpoint_url=ENDPOINT_URL,
-                                aws_access_key_id=ACCESS_KEY_ID,
-                                aws_secret_access_key=SECRET_ACCESS_KEY)
+            s3 = get_ceph_resource()
+            bucket_check = s3.Bucket(BUCKET_NAME) in s3.buckets.all()
+            
+            if bucket_check:
+                logger.info(f'{BUCKET_NAME} Bucket already exists!')
+            else:
+                logger.info(f'{BUCKET_NAME} Bucket does not exist!')
+                try:
+                    bucket = s3.Bucket(BUCKET_NAME)
+                    bucket.create()
+                except botocore.parsers.ResponseParserError as error:
+                #Bucket is created is successfully, but throwing an error. So that we are catching and passing that error(But not throwing that error)
+                    bucket_check = s3.Bucket(BUCKET_NAME) in s3.buckets.all()
+                    if bucket_check:
+                        logger.info(f'{BUCKET_NAME} bucket created...')
+                    else:
+                        logger.info(f'{BUCKET_NAME} bucket is not created...')
+                    pass
+            
             s3.Bucket(BUCKET_NAME).put_object(Bucket=BUCKET_NAME,
                                             Key=location,
                                             Body=content)
@@ -304,7 +331,8 @@ def generate_pdf(analysis_run):
         'route': '/full-view',
         'token': get_token(),
         'app_id': APP_ID,
-        'size': 'A4'
+        'size': 'A4',
+        'storage': storage_name
     }
 
     api_proce_before_time = time.time()
@@ -313,8 +341,8 @@ def generate_pdf(analysis_run):
     api_proce_diff = diff_sec(api_proce_before_time, api_proce_after_time)
     if api_proce_diff > API_TIME_STACKS:
         logging.info('Pdf response is took %d (greater than %d) seconds', api_proce_diff, API_TIME_STACKS)
-    logger.info('Pdf response is: %s', res['key'])
-    return res['key']
+    logger.info('Pdf response is: %s', res['Key'])
+    return res['Key']
 
 
 
@@ -421,11 +449,9 @@ def upload_excel_s3(local_file, bucket, s3_file):
 #Upload excel file to ceph
 def upload_excel_ceph(local_file, bucket, s3_file):
     try:
-        print("upload to excel file function in utilities")
-        s3 = get_s3_ceph()
+        s3 = get_ceph_resource()
         s3.Bucket(BUCKET_NAME).upload_file(Filename = local_file, Key = s3_file)
         logger.info('Upload successful')
-        
         delete_excel_file(local_file)
         return s3_file
     except FileNotFoundError:
@@ -442,7 +468,7 @@ def delete_excel_file(source_path):
         os.remove(source_path)
         logger.info('Excel file successfully deleted')
     except OSError as e:
-        logger.info(f'Failed to delete: %s : %s % {file_path, e.strerror}')
+        logger.info(f'Failed to delete: %s : %s % {source_path, e.strerror}')
 
 
 #Generate excel file
@@ -452,18 +478,40 @@ def generate_excel_file(run_id, orgId, report_gen_start_time, report_gen_complet
     reportname = f"{APP_ID.lower()}" + '-' + datetime.now().strftime('%Y-%m-%d-%I-%M-%S') + '.xlsx'
     filepath = './' + reportname
     generate_excel(run_id, orgId, excel_data, report_gen_start_time, report_gen_completed_time, filepath)
-    excel_file_location = f'{PLATFORM_ROUTE}/{run_id}/xls/' + reportname
+    # excel_file_location = f'{PLATFORM_ROUTE}/{run_id}/xls/' + reportname
     if storage_name == 's3':
+        excel_file_location = f'{PLATFORM_ROUTE}/{run_id}/xls/' + reportname
         excel_url = upload_excel_s3(filepath, BUCKET_NAME, excel_file_location)
     elif storage_name == 'ceph':
+        excel_file_location = f'{APP_ID.lower()}/{run_id}/xls/' + reportname
         excel_url = upload_excel_ceph(filepath, BUCKET_NAME, excel_file_location)
     else:
         logger.info("No storages are found")
     return excel_url
 
 
+def init_mail(org_id, run_id):
+    logger.info('Entered into init_mail method')
+    try:
+        url = BASE_API_URL + f'/analytics/api/v3/tenants/{org_id}/sends'
+        data={
+                "runId" : run_id
+            }
+        t1 = int(time.time())
+        res = call_post_requests(url , data=json.dumps(data), verify=False);
+        t2 = int(time.time())
+        duration = t2-t1
+        if duration > API_TIME_STACKS:
+            logging.info('Status update response took %d (greater than %d) seconds', duration, API_TIME_STACKS)
+        logger.info('Status update response is %s', res)
+    except Exception as e:
+        logger.error("Exception raised due to : " + repr(e))
+        traceback.print_exc()
+    logger.info('finished init_mail method')
+
+
 def upload_excel(analysis_run, excel_file):
     timestamp = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
     s3_path = f'{analysis_run.analysis.app.slug}/excel/{timestamp}.xlsx'
 
-    return upload_to_s3(excel_file, s3_path)
+    return upload_to_storage(excel_file, s3_path)
